# Simulation of Hosokawa and Tomiyama (2013)

## Description

Cases for low Reynolds number bubbly pipe flows (Re=900) have been set up using
the set of baseline models developed at HZDR for closure of all interfacial
forces involved in bubbly flows. The models are being validated with the
experimental data given in Hosokawa and Tomiyama (2013) [^1].

## Technical Information

The cases contain the experimental data for validation in the validation folder.
By executing the Allrun script inside the case folder all simulations are run
and plots are generated comparing the computed results to the experimental data.

## Pre-Processing

In order to simplify the CFD setup an axi-symmetric geometry (2D-wedge of
2.5 deg.) of the pipe is simulated. The CFD setup further assumes that both
phases enter from the same inlet with an inlet phase fraction derived from the
experiment and zero relative velocity between the phases.

## Post-Processing

See information inside cases.

### Data Validation

|  Field  | Validation | Reference |
|:-------:|:----------:|:---------:|
|  alpha  | alpha.dat  | line.csv  |
| k.water | liqKe.dat  | line.csv  |
| U.water | liqVel.dat | line.csv  |
|  U.air  | gasVel.dat | line.csv  |

## Keywords

pipe, circular-cross-section, constant-cross-section, bottom-injection,
vertical-upward, co-current, air-water, bubbly-flow, one-gas, fixed-bubble-size,
monodisperse, incompressible, tap-water, steady-state, fully-developed,
isothermal, 2D-wedge, two-phase-inlet, pressure-outlet, low-Re, low-alpha

## Additional Remarks

- Constant diameter model with bubble diameters given by the experiment
- Constant physical properties at standard conditions
- Zero velocity difference between phases at inlet, i.e. alpha=J_G/(J_G+J_L), U_G=U_L=J_G+J_L
- 1% turbulence intensity is prescribed at the inlet
- Turbulence length scale assumed to be 10% of the pipe diameter

## Literature

[^1]: Hosokawa, S., Tomiyama, A., 2013. Bubble-induced pseudo turbulence in
      laminar pipe flows. International Journal of Heat and Fluid Flow.
