# Simulation of Plunging Jet

## Description

This case is designed to test the performance of entrainment models. The
geometry is adapted from Chanson's plunging jet experimental setup[^1] to run on
a coarser mesh. A jet of water is entering the domain at the top (inlet diameter
and velocity are different than in Chanson's experiment), plunging into stagnant
water. A matching BC at the bottom removes water at the same rate as the inlet
to maintain the total volume of water in the domain. Castro's model is set as
default for entrainment. The case is monodisperse with bubble diameter
D = 3.5 mm.

## Technical Information

- 2D
- 18 600 cells
- 2.24 m x 2.4 m
- t < 2.5 s (computation time ~ 5 CPU hours on 16 CPUs)
- baseline model
- Porombka-Höhne interfacial drag model
- modified blending function psiSurfPhaseTransfer
- continuousBuoyancy (gravity term modification)

## Pre-Processing

Initialization of fields relies on the setFields utility.

## Post-Processing

Phase transfer rates 'dmdtf' for entrainment and detrainment are written as well
as blending factors 'f1' and 'f2' for the different blending methods. The field
'alpha.airD' is averaged over time, starting at t = 0.6 s.

## Keywords

2D, plunging-jet, isothermal, incompressible, air-water, entrainment,
detrainment, phase-transfer, top-inlet, vertical-downward, stagnant-water,
monodispersed, fixed-bubble-size, bubbly-flow, single-phase-inlet, atmosphere

## Additional remarks

The boundary conditions on the left and right sides are registered under the
name 'walls', but do not act as such. They are zeroGradient/inletOutlet BCs
instead. Changing the type from 'wall' to 'patch' makes the simulation crash
after t ~ 0.3 s. This behavior needs further investigation.

## Literature

[^1]: Chanson, H., Aoki, S. I., & Hoque, A. (2004).
      Physical modelling and similitude of air bubble entrainment at vertical
      circular plunging jets.
      Chemical engineering science, 59(4), 747-758.
